class AttackResult():
    """
    AttackResult class encapsulates the result of an adversarial attack, including the adversarial sample and the time taken to generate it.

    Attributes:
        _adv_sample (list): The adversarial sample generated by the attack, represented as a list of bytes.
        _elapsed_time (float): The time taken by the attack to generate the adversarial sample, measured in seconds.
    """

    def __init__(self, adv_sample, elapsed_time):
        """
        Initializes an AttackResult object with the given adversarial sample and elapsed time.

        Args:
            adv_sample (list): The adversarial sample generated by the attack, as a list of bytes.
            elapsed_time (float): The time taken by the attack to generate the adversarial sample, in seconds.
        """
        self._adv_sample = adv_sample
        self._elapsed_time = elapsed_time
    
    def get_adv_sample(self):
        """
        Returns the adversarial sample generated by the attack.

        Returns:
            The adversarial sample generated by the attack, as list of bytes.
        """
        return self._adv_sample
    
    def get_elapsed_time(self):
        """
        Returns the time taken by the attack to generate the adversarial sample.

        Returns:
            The time taken by the attack to generate the adversarial sample, in seconds.
        """
        return self._elapsed_time

    