from ember.features import PEFeatureExtractor

def get_ember_features_from_file(path):
    """
    Extracts a feature vector from a binary file using the ember Feature Extractor.

    Args:
        path (str): The path to the binary file.

    Returns:
        list: A list of float values representing the feature vector.
    """
    with open(path, 'rb') as f:
        extractor = PEFeatureExtractor(print_feature_warning=False)
        bytes = f.read()
        features = [float(value) for value in extractor.feature_vector(bytes)]
        return features
    
def get_ember_features_from_bytes(bytes):
    """
    Extracts a feature vector from a binary file bytes using the ember Feature Extractor.

    Args:
        bytes (bytes): The bytes of the binary file.

    Returns:
        list: A list of float values representing the feature vector.
    """
    extractor = PEFeatureExtractor(print_feature_warning=False)
    features = [float(value) for value in extractor.feature_vector(bytes)]
    return features